#! /usr/bin/env python


#############################################################################
##                                                                         ##
## shellforge.py --- C to shellcode conversion programm                    ##
##              see http://www.cartel-securite.net/pbiondi/shellforge.html ##
##              for more informations                                      ##
##                                                                         ##
## Copyright (C) 2003  Philippe Biondi <biondi@cartel-securite.fr>         ##
##                                                                         ##
## This program is free software; you can redistribute it and/or modify it ##
## under the terms of the GNU General Public License as published by the   ##
## Free Software Foundation; either version 2, or (at your option) any     ##
## later version.                                                          ##
##                                                                         ##
## This program is distributed in the hope that it will be useful, but     ##
## WITHOUT ANY WARRANTY; without even the implied warranty of              ##
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       ##
## General Public License for more details.                                ##
##                                                                         ##
#############################################################################

# Inspired from stealth@TESO's HellKit

import os,sys,re,getopt


RCSID="$Id: shellforge.py,v 1.5 2003/05/11 15:28:46 biondi Exp $"
MAJORVERSION="0"
VERSION = MAJORVERSION+"."+RCSID.split()[2]


VERB=1
C_OUTPUT=0
TEST=0
KEEP=0


def printinfo(lvl, x):
    if lvl <= VERB:
        os.write(2,x+"\n")

def error(x,err=1,printusage=0):
    printinfo(0,"ERROR: %s" % x)
    if printusage:
        usage()
    sys.exit(err)
        

def usage():
    printinfo(0, """Usage: shellforge [-v <verbosity>] [-c] [-t[t]] [-k[k]] <src.c>
  -v <verb> : adjust verbosity. Default is 1.
  -V        : return version number.
  -c        : ouput c instead of raw shellcode.
  -t        : output the code to a file (.tst.c) and compile it (imply -c).
  -tt       : same as -t, then try to run it.
  -k        : keep intermediate files (.s and .o, or .tst and tst.c with -tt).
  -k        : even keep .s and .o if in -tt mode.""")

try:
    opts=getopt.getopt(sys.argv[1:], "hv:ctkV")
    for opt,parm in opts[0]:
        if opt == "-h":
            usage()
            sys.exit(0)
        if opt == "-V":
            printinfo(0, "ShellForge v%s\nCopyright (C) 2003  Philippe Biondi <biondi@cartel-securite.fr>" % VERSION)
            sys.exit(0)
        elif opt == "-c":
            C_OUTPUT = 1
        elif opt == "-t":
            TEST += 1
        elif opt == "-k":
            KEEP += 1
        elif opt == "-v":
            try:
                VERB = int(parm)
            except ValueError,msg:
                raise getopt.GetoptError(str(msg),None)
    if len(opts[1]) > 1:
        raise getopt.GetoptError("too many paramters after options.",None)
    elif len(opts[1]) == 0:
        raise getopt.GetoptError("source file missing.",None)
    src = opts[1][0]
except getopt.GetoptError,msg:
    error(msg, printusage=1)



def mkdecryptloader(length, xorkey):
    """Loader code is 
main:
	jmp .l2
.l1:
	pop %esi
	xorl %ecx, %ecx
	movw $0x1234, %cx
.loop:	
	xorb $0x56, (%esi)
	incl %esi
	loop .loop
.l2:
	call .l1
#SHELLCODE HERE	
"""
    if length < 0x100:
        length = 0x101
    if length & 0xff == 0:
        length += 1
    return ("\xeb\x0f\x5e\x31\xc9\x66\xb9"+chr(length&0xff)+chr(length>>8)+
            "\x80\x36"+chr(xorkey)+"\x46\xe2\xfa\xeb\x05\xe8\xec\xff\xff\xff")



printinfo(1,"** Compiling %s" % src)

f=os.popen("gcc -O3 -S -fPIC -Winline -finline-functions  -ffreestanding -o -  %s" % src)
inp = f.readlines()
if f.close():
    error("gcc reported error while compiling %s" % src, err=2)

printinfo(1,"** Tuning original assembler code")

# Move .rodata at the end of .text
# Move absolute address retrieval at the begining
# Make %esp point at the begining of the shellcode right after that
# Put frame stack code after that
out = []
st1 = []
st2 = []
state=0
for l in inp:
    printinfo(2, "[%i] %s"% (state, l[:-1]))
    if l.find("@PLT") >= 0:
        error("Error at [%s]: Symbol not found" % (l.strip()), err=2)
    if state == 0:
        if l.find(".rodata") >= 0:
	    state = 1
	else:
	    out.append(l)
    elif state == 1:
        if l.startswith(".text"):
            out.append(l)
            state = 2
        else:
            st1.append(l)
    elif state == 2:
        out.append(l)
        if l.startswith("main:"):
            state = 3
    elif state == 3:
        if l.find("call") >= 0:
            out.append(l)
            state=4
        else:
            st2.append(l)
    elif state == 4:
        if l.find("GLOBAL_OFFSET_TABLE") >= 0:
            out.append("\tsubl $5,%ebx\n")
            out.append("\tmovl %ebx, %eax\n")
            out.append("\txorl %esp, %eax\n")
            out.append("\tshrl $16, %eax\n")
            out.append("\ttest %eax, %eax\n")                       
            out.append("\tjnz .Lnotinstack\n")                       
            out.append("\tmovl main@GOTOFF(%ebx),%esp\n")
            out.append(".Lnotinstack:\n")
            out += st2
            st2 = []
            state = 5
        else:
            out.append(l)

    elif state == 5:
        if l.startswith(".Lfe1:"):
            out += st1
            st1 = []
        out.append(l)

f=open(src+".s","w")
f.writelines(out)
f.close()

printinfo(1,"** Assembling modified asm")
		
status = os.system("gcc -c -o %s.o %s.s" % (src,src))
if status != 0:
    error("gcc reported error while compiling modified asm", err=2)

printinfo(1,"** Retrieving machine code")

f=os.popen("objdump -j .text -s -z  %s.o" % src)
inp = f.readlines()
f.close()


# Extract machine code
dump=re.compile("^ [0-9a-f]{4}")
out = []
for l in inp:
    if dump.match(l):
        out += l[:42].split()[1:]
out = "".join(out)
shcode = ""
for i in range(len(out)/2):
    shcode += chr(int(out[2*i:2*i+2],16))

if not shcode:
    error("No code in .text section of %s.o !?"%src,err=2)

printinfo(1, "** Computing xor encryption key")

key=0
for i in range(1,256):
    if chr(i) not in shcode:
        key=i
        break
if key == 0:
    printinfo(0,"Error: no suitable xor key found.")
    printinfo(0,"Try a better shellcode mutation algorithm on this :")
else:
    shcode = "".join(map(lambda x: chr(ord(x)^key), shcode))
    shcode = mkdecryptloader(len(shcode),key)+shcode
    printinfo(1,"** Shellcode forged!")

if TEST:
    fd = os.open("%s.tst.c"% src, os.O_WRONLY|os.O_CREAT)
    C_OUTPUT = 1
else:
    fd = 1

if C_OUTPUT:
    os.write(fd, 'unsigned char shellcode[] = \n"')
for i in range(len(shcode)):
    os.write(fd, "\\x%02x" % ord(shcode[i]))
    if C_OUTPUT and i%19 == 18:
        os.write(fd,'"\n"')
if C_OUTPUT:
    os.write(fd,'";\nint main(void) { ((void (*)())shellcode)(); }')
os.write(fd,"\n")

if TEST:
    printinfo(1, "** Compiling test program")
    status = os.system("gcc -o %s.tst %s.tst.c" % (src,src))
    if status != 0:
        error("gcc reported error while compiling test program",err=3)
    if TEST > 1:
        printinfo(1, "** Running test program")
        abs=""
        if src[0] != "/":
            abs = "./"
        status=os.system(abs+src+".tst")
        printinfo(1, "\n** Test done! Returned status=%i"% (status>>8))
    else:
        printinfo(1, "** Test program (%s.tst) compiled!" % src)

if TEST == 0 or KEEP < TEST:
    os.unlink(src+".s")
    os.unlink(src+".o")
if TEST == 2 and not KEEP:
    os.unlink(src+".tst.c")
    os.unlink(src+".tst")

